@@ -1,5 +1,5 @@
 /*
- * written by J. Marcin Gorycki <mgo@olicom.dk>
+ * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ GnomeCanvasItem *BlockOps::generateItem(int x, int y, int color)
   			"width", (double) BLOCK_SIZE,
   			"height", (double) BLOCK_SIZE,
   			"anchor", GTK_ANCHOR_NW,
-  			NULL);
+  			0);
 
 	return it;
 }
@@ -46,6 +46,7 @@ BlockOps::BlockOps(Field *f)
 	for (int i = 0; i < COLUMNS; ++i)
 		field[i] = new Block[LINES];
 
+	field_initialized = false;
 	emptyField();
 }
 
@@ -137,14 +138,21 @@ BlockOps::moveBlockRight()
 }
 
 bool
-BlockOps::rotateBlock()
+BlockOps::rotateBlock(bool rotateCCW)
 {
 	bool moved = false;
 	
 	int r = rot;
-	if (++r >= 4)
-		r = 0;
 	
+	if ( rotateCCW )
+	{
+		if (--r < 0) r = 3;
+	}
+	else
+	{
+		if (++r >= 4) r = 0;
+	}
+		
 	if (blockOkHere(posx, posy, blocknr, r))
 	{
 		putBlockInField(true);
@@ -226,7 +234,7 @@ BlockOps::checkFullLines(ScoreFrame *s)
 						if (field[x][y1].item)
 						{
 							gtk_object_destroy(GTK_OBJECT(field[x][y1].item));
-							field[x][y1].item = NULL;
+							field[x][y1].item = 0;
 							if (y1 < y)
 								field[x][y1 + 1].item = generateItem(x, y1 + 1, field[x][y1 + 1].color);
 						}
@@ -262,20 +270,43 @@ BlockOps::generateFallingBlock()
 }
 
 void
-BlockOps::emptyField()
+BlockOps::emptyField(int filled_lines, int fill_prob)
 {
+	int blank;
+
 	for (int y = 0; y < LINES; ++y)
 	{
+		blank = rand() % COLUMNS; // Allow for at least one blank per line
 		for (int x = 0; x < COLUMNS; ++x)
 		{
-			if (field[x][y].item != NULL)
+			if (field_initialized && field[x][y].item != 0)
 			{
 				gtk_object_destroy(GTK_OBJECT(field[x][y].item));
-				field[x][y].item = NULL;
 			}
+			field[x][y].item = 0;
 			field[x][y].what = EMPTY;
+			if ((y>=(LINES - filled_lines)) && (x != blank) &&
+			    ((rand() % 10) < fill_prob)) { 
+				field[x][y].what = LAYING;
+				if (nr_of_colors)
+					field[x][y].color = rand() % nr_of_colors;
+				else
+					// This is in case we're called
+					// before the widgets are set 
+					// up and nr_of_colors is
+					// defined.
+					field[x][y].color = 0; 
+				field[x][y].item = generateItem(x,y,field[x][y].color);
+			}
 		}
 	}
+	field_initialized = true;
+}
+
+void
+BlockOps::emptyField(void)
+{
+	emptyField(0,5);
 }
 
 void
@@ -289,7 +320,7 @@ BlockOps::putBlockInField(bool erase)
 				if (erase)
 				{
 					gtk_object_destroy(GTK_OBJECT(field[posx - 2 + x][y + posy].item));
-					field[posx - 2 + x][y + posy].item = NULL;
+					field[posx - 2 + x][y + posy].item = 0;
 				}
 				else
 				{
