@@ -1,5 +1,5 @@
 /*
- * written by J. Marcin Gorycki <mgo@olicom.dk>
+ * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <config.h>
 #include "tetris.h"
 
-int nr_of_colors;
+int nr_of_colors = 0;
 
 int
 main(int argc, char *argv[])
@@ -34,16 +34,16 @@ main(int argc, char *argv[])
 	poptOption options[] = 
 	{
 		{"level", 'l', POPT_ARG_INT, &cmdlineLevel, 0, N_("Set starting level (1-10)"), N_("LEVEL")},
-		{NULL, '\0', 0, NULL, 0}
+		{0, '\0', 0, 0, 0}
 	};
 
-	srand(time(NULL));
+	srand(time(0));
 
 	gnome_score_init("gnometris");
 
-	gnome_init_with_popt_table("gnometris", TETRIS_VERSION, argc, argv, options, 0, NULL);
+	gnome_init_with_popt_table("gnometris", TETRIS_VERSION, argc, argv, options, 0, 0);
 
-	GnomeClient *client= gnome_master_client();
+//	GnomeClient *client= gnome_master_client();
 	
 	Tetris * t = new Tetris(cmdlineLevel);
 
