@@ -1,5 +1,5 @@
 /*
- * written by J. Marcin Gorycki <mgo@olicom.dk>
+ * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,25 +51,26 @@ int color_next = -1;
 
 bool random_block_colors = false;
 bool do_preview = true;
+bool rotateCounterClockWise = true;
 
-char *Tetris::blockPixmapTmp = NULL;
-char *Tetris::bgPixmapTmp = NULL;
+char *Tetris::blockPixmapTmp = 0;
+char *Tetris::bgPixmapTmp = 0;
 
 Tetris::Tetris(int cmdlLevel): 
+	blockPixmap(0),
+	bgPixmap(0),
+	field(0),
+	preview(0),
+	scoreFrame(0),
 	paused(false), 
 	timeoutId(-1), 
 	onePause(false), 
-	setupdialog(NULL), 
+	image(0),
+	bgimage(0),
+	setupdialog(0), 
 	cmdlineLevel(cmdlLevel), 
 	doPreviewTmp(do_preview), 
 	randomBlocksTmp(random_block_colors),
-	blockPixmap(NULL),
-	bgPixmap(NULL),
-	image(NULL),
-	bgimage(NULL),
-	scoreFrame(NULL),
-	field(NULL),
-	preview(NULL),
 	fastFall(false)
 {
 	pic = new GdkImlibImage*[tableSize];
@@ -117,8 +118,11 @@ Tetris::Tetris(int cmdlLevel):
 		GNOMEUIINFO_END
 	};
 
-	blockPixmap = strdup(gnome_config_get_string_with_default("/gnometris/Properties/BlockPixmap=7blocks-tig.png", NULL));
-	bgPixmap = strdup(gnome_config_get_string_with_default("/gnometris/Properties/BackgroundPixmap=fishy-bg.png", NULL));
+	line_fill_height = 0;
+	line_fill_prob = 5;
+
+	blockPixmap = strdup(gnome_config_get_string_with_default("/gnometris/Properties/BlockPixmap=7blocks-tig.png", 0));
+	bgPixmap = strdup(gnome_config_get_string_with_default("/gnometris/Properties/BackgroundPixmap=fishy-bg.png", 0));
 
 
 	gnome_app_create_menus(GNOME_APP(w), mainmenu);
@@ -190,7 +194,7 @@ Tetris::setupdialogDestroy(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	gtk_widget_destroy(t->setupdialog);
-	t->setupdialog = NULL;
+	t->setupdialog = 0;
 }
 
 void
@@ -198,7 +202,7 @@ Tetris::setupPixmap()
 {
 	char *pixname, *fullpixname;
 	
-	pixname = g_copy_strings("gnometris/", blockPixmap, NULL);
+	pixname = g_copy_strings("gnometris/", blockPixmap, 0);
 	fullpixname = gnome_unconditional_pixmap_file(pixname);
 	g_free(pixname);
 
@@ -226,7 +230,7 @@ Tetris::setupPixmap()
 		pic[i] = gdk_imlib_crop_and_clone_image(image, (i % nr_of_colors) * BLOCK_SIZE, 0, BLOCK_SIZE, BLOCK_SIZE);
 	}
 
-	pixname = g_copy_strings("gnometris/bg/", bgPixmap, NULL);
+	pixname = g_copy_strings("gnometris/bg/", bgPixmap, 0);
 	fullpixname = gnome_unconditional_pixmap_file(pixname);
 	g_free(pixname);
 
@@ -236,18 +240,18 @@ Tetris::setupPixmap()
 	if (g_file_exists(fullpixname)) 
 		bgimage = gdk_imlib_load_image(fullpixname);
 	else
-		bgimage = NULL;
+		bgimage = 0;
 
 	if (field)
 	{
 		field->updateSize(bgimage);
-		gtk_widget_draw(field->getWidget(), NULL);
+		gtk_widget_draw(field->getWidget(), 0);
 	}
 	
 	if (preview)
 	{
 		preview->updateSize();
-		gtk_widget_draw(preview->getWidget(), NULL);
+		gtk_widget_draw(preview->getWidget(), 0);
 	}
 
 	// FIXME: this really sucks, but I can't find a better way to resize 
@@ -267,12 +271,18 @@ Tetris::doSetup(GtkWidget *widget, void *d)
 	Tetris *t = (Tetris*) d;
 	t->cmdlineLevel = 0;
 	t->startingLevel = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(t->sentry));
+	t->line_fill_prob = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(t->fill_prob_spinner));
+	t->line_fill_height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(t->fill_height_spinner));
 	do_preview = t->doPreviewTmp;
 	random_block_colors = t->randomBlocksTmp;
+ 	rotateCounterClockWise = t->rotateCounterClockWiseTmp;
 	
 	gnome_config_set_int("/gnometris/Properties/StartingLevel", t->startingLevel);
 	gnome_config_set_int("/gnometris/Properties/DoPreview", do_preview);
 	gnome_config_set_int("/gnometris/Properties/RandomBlockColors", random_block_colors);
+ 	gnome_config_set_int("/gnometris/Properties/RotateCounterClockWise", rotateCounterClockWise);
+	gnome_config_set_int("/gnometris/Properties/LineFillHeight", t->line_fill_height);
+	gnome_config_set_int("/gnometris/Properties/LineFillProbability", t->line_fill_prob);
 	
 	if (blockPixmapTmp)
 	{
@@ -311,6 +321,13 @@ Tetris::setSelectionBlocks(GtkWidget *widget, void *d)
 	t->randomBlocksTmp = GTK_TOGGLE_BUTTON(widget)->active;
 }
 
+void 
+Tetris::setRotateCounterClockWise(GtkWidget *widget, void *d)
+{
+	Tetris *t = (Tetris*) d;
+	t->rotateCounterClockWiseTmp = GTK_TOGGLE_BUTTON(widget)->active;
+}
+
 void
 Tetris::setSelection(GtkWidget *widget, void *data)
 {
@@ -347,7 +364,7 @@ Tetris::fillMenu(GtkWidget *menu, char *pixname, char *dirname, GtkSignalFunc se
 	GtkWidget *item;
 	char *s;
 	
-	while ((e = readdir (dir)) != NULL)
+	while ((e = readdir (dir)) != 0)
 	{
 		s = strdup(e->d_name);
 
@@ -388,7 +405,8 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 {
 	GtkWidget *allBoxes;
 	GtkWidget *box, *box2;
-	GtkWidget *button, *label;
+//	GtkWidget *button;
+	GtkWidget *label;
 	GtkWidget *frame;
 	GtkObject *adj;
         
@@ -397,10 +415,14 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	if (t->setupdialog) 
 		return FALSE;
 
+
+	t->line_fill_height = gnome_config_get_int_with_default("/gnometris/Properties/LineFillHeight=0",0);
+	t->line_fill_prob = gnome_config_get_int_with_default("/gnometris/Properties/LineFillProbability=5",0);
+
 	t->setupdialog = gnome_dialog_new(_("Gnometris setup"), 
 																		GNOME_STOCK_BUTTON_OK, 
 																		GNOME_STOCK_BUTTON_CANCEL, 
-																		NULL);
+																		0);
 	gnome_dialog_set_parent(GNOME_DIALOG(t->setupdialog), GTK_WINDOW(t->w));
 
 	gtk_container_border_width(GTK_CONTAINER(t->setupdialog), 10);
@@ -420,15 +442,58 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	gtk_widget_show(frame);
 
 	box2 = gtk_hbox_new(FALSE, 0);
-  gtk_container_border_width(GTK_CONTAINER(box2), 10);
+	gtk_container_border_width(GTK_CONTAINER(box2), 10);
+	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+	label = gtk_label_new(_("Number of pre-filled rows:"));
+	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
+	gtk_widget_show(label);
+	adj = gtk_adjustment_new(t->line_fill_height,0,LINES-1,1,5,0);
+	t->fill_height_spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj),10,0);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(t->fill_height_spinner),
+					  GTK_UPDATE_ALWAYS
+#ifndef HAVE_GTK_SPIN_BUTTON_SET_SNAP_TO_TICKS
+					  | GTK_UPDATE_SNAP_TO_TICKS
+#endif
+		);
+#ifdef HAVE_GTK_SPIN_BUTTON_SET_SNAP_TO_TICKS
+	gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(t->fill_height_spinner), 1);
+#endif
+	gtk_box_pack_end(GTK_BOX(box2), t->fill_height_spinner, 0, 0, 0);
+	gtk_widget_show(t->fill_height_spinner);	
+	gtk_widget_show(box2);
+
+	box2 = gtk_hbox_new(FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(box2), 10);
+	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
+	label = gtk_label_new(_("Density of blocks in a pre-filled row:"));
+	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
+	gtk_widget_show(label);
+	adj = gtk_adjustment_new(t->line_fill_prob,0,10,1,5,0);
+	t->fill_prob_spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj),10,0);
+	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(t->fill_prob_spinner),
+					  GTK_UPDATE_ALWAYS
+#ifndef HAVE_GTK_SPIN_BUTTON_SET_SNAP_TO_TICKS
+					  | GTK_UPDATE_SNAP_TO_TICKS
+#endif
+		);
+#ifdef HAVE_GTK_SPIN_BUTTON_SET_SNAP_TO_TICKS
+	gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(t->fill_prob_spinner), 1);
+#endif
+	gtk_box_pack_end(GTK_BOX(box2), t->fill_prob_spinner, 0, 0, 0);
+	gtk_widget_show(t->fill_prob_spinner);	
+	gtk_widget_show(box2);
+
+	box2 = gtk_hbox_new(FALSE, 0);
+	gtk_container_border_width(GTK_CONTAINER(box2), 10);
 	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
 	label = gtk_label_new(_("Starting Level:"));
 	gtk_box_pack_start(GTK_BOX(box2), label, 0, 0, 0);
 	gtk_widget_show(label);
 
-  t->startingLevel = gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", NULL);
-	random_block_colors = gnome_config_get_int_with_default("/gnometris/Properties/RandomBlockColors=0", NULL) != 0;
-	do_preview = gnome_config_get_int_with_default("/gnometris/Properties/DoPreview=0", NULL) != 0;
+  t->startingLevel = gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", 0);
+	random_block_colors = gnome_config_get_int_with_default("/gnometris/Properties/RandomBlockColors=0", 0) != 0;
+	do_preview = gnome_config_get_int_with_default("/gnometris/Properties/DoPreview=0", 0) != 0;
+ 	rotateCounterClockWise = gnome_config_get_int_with_default("/gnometris/Properties/RotateCounterClockWise=0", NULL) != 0;
 
   adj = gtk_adjustment_new(t->startingLevel, 1, 10, 1, 5, 10);
 	t->sentry = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 10, 0);
@@ -457,7 +522,13 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	gtk_box_pack_start(GTK_BOX(box), cb, 0, 0, 0);
 	gtk_widget_show(cb);
 
-	blockPixmapTmp=NULL;
+ 	cb = gtk_check_button_new_with_label(_("Rotate counterclockwise"));
+ 	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cb), rotateCounterClockWise);
+ 	gtk_signal_connect(GTK_OBJECT(cb), "clicked", (GtkSignalFunc)setRotateCounterClockWise, d);
+ 	gtk_box_pack_start(GTK_BOX(box), cb, 0, 0, 0);
+ 	gtk_widget_show(cb);
+
+	blockPixmapTmp=0;
 	box2 = gtk_hbox_new(FALSE, 0);
   gtk_container_border_width(GTK_CONTAINER(box2), 10);
 	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
@@ -473,7 +544,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 	gtk_box_pack_start(GTK_BOX(box), box2, 0, 0, 0);
 	gtk_widget_show(box2);
 
-	bgPixmapTmp=NULL;
+	bgPixmapTmp=0;
 	box2 = gtk_hbox_new(FALSE, 0);
   gtk_container_border_width(GTK_CONTAINER(box2), 10);
 	gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
@@ -498,6 +569,7 @@ Tetris::gameProperties(GtkWidget *widget, void *d)
 															GTK_SIGNAL_FUNC(setupdialogDestroy), d);
 	
 	gtk_widget_show(t->setupdialog);
+	return TRUE;
 }
 
 int
@@ -505,6 +577,7 @@ Tetris::gamePause(GtkWidget *widget, void *d)
 {
 	Tetris *t = (Tetris*) d;
 	t->togglePause();
+	return TRUE;
 }
 
 int
@@ -516,6 +589,7 @@ Tetris::gameEnd(GtkWidget *widget, void *d)
 	t->timeoutId = -1;
 	blocknr_next = -1;
 	t->endOfGame();
+	return TRUE;
 }
 
 int
@@ -566,12 +640,12 @@ Tetris::timeoutHandler(void *d)
  	if (t->onePause)
  	{
 		t->onePause = false;
-		gtk_widget_draw(t->field->getWidget(), NULL);
+		gtk_widget_draw(t->field->getWidget(), 0);
 	}
  	else
 	{
 		bool res = t->ops->moveBlockDown();
-		gtk_widget_draw(t->field->getWidget(), NULL);
+		gtk_widget_draw(t->field->getWidget(), 0);
 
 		if (res)
 		{
@@ -622,7 +696,7 @@ Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 			res = t->ops->moveBlockRight();
 			break;
 		case GDK_Up:
-			res = t->ops->rotateBlock();
+			res = t->ops->rotateBlock(rotateCounterClockWise);
 			break;
 		case GDK_Down:
 			if (!t->fastFall)
@@ -657,7 +731,7 @@ Tetris::eventHandler(GtkWidget *widget, GdkEvent *event, void *d)
 	}
 
 	if (res)
-		gtk_widget_draw(t->field->getWidget(), NULL);
+		gtk_widget_draw(t->field->getWidget(), 0);
 
 	return (keyEvent == true);
 }
@@ -674,7 +748,7 @@ Tetris::generate()
 	if (ops->generateFallingBlock())
 	{
 		ops->putBlockInField(false);
-		gtk_widget_draw(preview->getWidget(), NULL);
+		gtk_widget_draw(preview->getWidget(), 0);
 		onePause = true;
 	}
 	else
@@ -699,7 +773,7 @@ Tetris::endOfGame()
 	
 	if (scoreFrame->getScore() > 0) 
 	{
-		int pos = gnome_score_log(scoreFrame->getScore(), NULL, TRUE);
+		int pos = gnome_score_log(scoreFrame->getScore(), 0, TRUE);
 		showScores("Gnometris", pos);
 	}
 }
@@ -707,7 +781,7 @@ Tetris::endOfGame()
 void
 Tetris::showScores(gchar *title, guint pos)
 {
-	gnome_scores_display(title, "gnometris", NULL, pos);
+	gnome_scores_display(title, "gnometris", 0, pos);
 }
 
 int
@@ -721,26 +795,28 @@ Tetris::gameNew(GtkWidget *widget, void *d)
 		t->timeoutId = -1;
 
 		if (t->scoreFrame->getScore() > 0) 
-			gnome_score_log(t->scoreFrame->getScore(), NULL, TRUE);
+			gnome_score_log(t->scoreFrame->getScore(), 0, TRUE);
 	}
 
 	t->fastFall = false;
 	
   int level = t->cmdlineLevel ? t->cmdlineLevel :
-		gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", NULL);
+		gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", 0);
 	t->scoreFrame->setLevel(level);
-	random_block_colors = gnome_config_get_int_with_default("/gnometris/Properties/RandomBlockColors=0", NULL) != 0;
-	do_preview = gnome_config_get_int_with_default("/gnometris/Properties/DoPreview=0", NULL) != 0;
-	
+	random_block_colors = gnome_config_get_int_with_default("/gnometris/Properties/RandomBlockColors=0", 0) != 0;
+	do_preview = gnome_config_get_int_with_default("/gnometris/Properties/DoPreview=0", 0) != 0;
+	t->line_fill_height = gnome_config_get_int_with_default("/gnometris/Properties/LineFillHeight=0",0);
+	t->line_fill_prob = gnome_config_get_int_with_default("/gnometris/Properties/LineFillProbability=5",0);
+
 	t->timeoutId = gtk_timeout_add(1000 - 100 * (level - 1), timeoutHandler, t);
-	t->ops->emptyField();
+	t->ops->emptyField(t->line_fill_height,t->line_fill_prob);
 
 	t->scoreFrame->resetLines();
 	t->paused = false;
 	
 	t->ops->generateFallingBlock();
-	gtk_widget_draw(t->field->getWidget(), NULL);
-	gtk_widget_draw(t->preview->getWidget(), NULL);
+	gtk_widget_draw(t->field->getWidget(), 0);
+	gtk_widget_draw(t->preview->getWidget(), 0);
 
 	gtk_widget_set_sensitive(t->gameMenuPtr[1].widget, TRUE);
 	gtk_widget_set_sensitive(t->gameMenuPtr[4].widget, TRUE);
@@ -756,12 +832,12 @@ Tetris::gameAbout(GtkWidget *widget, void *d)
 
 	GtkWidget *about;
 
-	const gchar *authors[] = {"J. Marcin Gorycki", NULL};
+	const gchar *authors[] = {"J. Marcin Gorycki", 0};
 
 	about = gnome_about_new("Gnometris", TETRIS_VERSION, "(C) 1999 J. Marcin Gorycki", 
 													(const char **)authors,
 													_("Written for my wife, Matylda\n"
-														"Send comments and bug reports to: mgo@olicom.dk"), NULL);
+														"Send comments and bug reports to: marcin.gorycki@intel.com"), 0);
 	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(t->w));
 	gtk_window_set_modal(GTK_WINDOW(about), TRUE);
 
