@@ -398,7 +398,7 @@ void about_cb(GtkWidget *widget, gpointer data)
 	}
 
 	about = gnome_about_new(_("Iagno"), VERSION, _("(C) 1998 Ian Peters"),
-			(const char **)authors, _("Send comments and bug reports to: itp@acm.org\nTiles under the General Public License."), NULL);
+			(const char **)authors, _("Send comments and bug reports to: itp@gnu.org\nTiles under the General Public License."), NULL);
 	gtk_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
 			(gtk_widget_destroyed), &about);
 	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(window));
@@ -591,6 +591,16 @@ void init_new_game()
 		flip_final_id = 0;
 	}
 
+	if (black_computer_id) {
+		gtk_timeout_remove(black_computer_id);
+		black_computer_id = 0;
+	}
+
+	if (white_computer_id) {
+		gtk_timeout_remove(white_computer_id);
+		white_computer_id = 0;
+	}
+
 	game_in_progress = 1;
 	move_count = 0;
 
