@@ -28,7 +28,10 @@
 
 #define APPNAME "mahjongg"
 #define APPNAME_LONG "Gnome Mahjongg"
+/*
 #define MAH_VERSION "0.99.2+"
+*/
+#define MAH_VERSION VERSION
 
 #define AREA_WIDTH 600
 #define AREA_HEIGHT 470
@@ -879,7 +882,7 @@ void no_match (void)
 	if (popup_config.warn.popup) {
                 mb = gnome_message_box_new (_("Tiles don't match!"),
                                             GNOME_MESSAGE_BOX_INFO,
-                                            _("Ok"), NULL);
+                                            GNOME_STOCK_BUTTON_OK, NULL);
                 GTK_WINDOW (mb)->position = GTK_WIN_POS_MOUSE;
                 gtk_window_set_modal (&GNOME_MESSAGE_BOX(mb)->dialog.window, TRUE);
                 
@@ -912,7 +915,7 @@ void check_free (void)
                 if (!game_over) {
                         mb = gnome_message_box_new (_("No more moves"), 
                                                     GNOME_MESSAGE_BOX_INFO, 
-                                                    _("Ok"), NULL); 
+                                                    GNOME_STOCK_BUTTON_OK, NULL); 
                         GTK_WINDOW (mb)->position = GTK_WIN_POS_MOUSE; 
                         gtk_window_set_modal (&GNOME_MESSAGE_BOX (mb)->dialog.window, TRUE);
                         gnome_dialog_set_parent(&GNOME_MESSAGE_BOX (mb)->dialog, GTK_WINDOW (window));
@@ -1390,7 +1393,7 @@ void about_callback (GtkWidget *widget, gpointer data)
 				 (const char **)authors,
 				 _("Send comments and bug reports to:\n"
 				   "        pancho@nuclecu.unam.mx or\n"
-				   "        michael@imaginator.com\n\n"
+				   "        mmeeks@gnu.org\n\n"
 				   "Tiles under the General Public License."),
 				 NULL);
 
@@ -1923,7 +1926,7 @@ void shuffle_tiles_callback (GtkWidget *widget, gpointer data)
         tile temp;
         time_t seconds;
 
-        if (paused || game_over == GAME_DEAD) return;
+        if (paused || game_over == GAME_DEAD || game_over == GAME_WON) return;
 
         do {
                 num_shuffle++;
@@ -1952,7 +1955,7 @@ void shuffle_tiles_callback (GtkWidget *widget, gpointer data)
                 gtk_clock_stop (GTK_CLOCK (chrono));
                 mb = gnome_message_box_new (_("Sorry, I can't find\na playable configuration."), 
                                             GNOME_MESSAGE_BOX_INFO, 
-                                            _("Ok"), NULL); 
+                                            GNOME_STOCK_BUTTON_OK, NULL); 
                 GTK_WINDOW (mb)->position = GTK_WIN_POS_MOUSE; 
                 gtk_window_set_modal (&GNOME_MESSAGE_BOX (mb)->dialog.window, TRUE);
                 gnome_dialog_set_parent(&GNOME_MESSAGE_BOX (mb)->dialog, GTK_WINDOW (window));
